@@ -114,7 +114,21 @@ token_queue_refresh_rate = 0.01
 # Collect the amount of handed out tokens in the last ↓ secs for measuring the bandwidth
 token_queue_download_refresh_rate = 3
 
-ffmpeg_args = ["-crf", "35", "-c:v", "libx265", "-c:a", "copy", "-preset", "fast"]
+ffmpeg_args = ["-crf", "33", "-c:v", "libx265", "-c:a", "copy", "-preset", "superfast"]
+
+# If the efficiency is too low for ↓ seconds the file will be blacklisted from compression
+compress_duration_for_to_low_efficiency = 0.5
+
+
+compress_minimum_stdev = 0.5
+
+compress_score_mavg_size = 5
+compress_std_mavg_size = 5
+
+compress_minimum_score = 1.6
+compress_insta_kill_score = 1.9
+
+compress_duration_for_insta_kill = 0
 
 
 # Now load any options the user may overwrite (Linux exclusive)
